import { TreeRenderResultFlatted } from '@/lib/XMLProcessor/src/typing'
import { cloneDeep } from 'lodash'

export function dualCompare(oldItems: TreeRenderResultFlatted[], newItems: TreeRenderResultFlatted[]) {
    const oldItemsCount = oldItems.length
    const newItemsCount = newItems.length

    const dp: number[][] = Array.from({ length: oldItemsCount + 1 }, () => Array(newItemsCount + 1).fill(0))

    for (let i = 1; i <= oldItemsCount; i++) {
        for (let j = 1; j <= newItemsCount; j++) {
            if (oldItems[i - 1].hash === newItems[j - 1].hash) {
                dp[i][j] = dp[i - 1][j - 1] + 1
            } else {
                dp[i][j] = Math.max(dp[i - 1][j], dp[i][j - 1])
            }
        }
    }

    let i = oldItemsCount,
        j = newItemsCount
    const result: Array<{ type: 'same' | 'deleted' | 'added' | 'changedOld' | 'changedNew'; item: TreeRenderResultFlatted }> = []
    while (i > 0 && j > 0) {
        if (oldItems[i - 1].hash === newItems[j - 1].hash) {
            // result.unshift({ type: 'same', item: cloneDeep(oldItems[i - 1]) })
            i--
            j--
        } else if (dp[i - 1][j] > dp[i][j - 1]) {
            result.unshift({
                type: 'deleted',
                item: cloneDeep(oldItems[i - 1])
            })
            i--
        } else {
            result.unshift({
                type: 'added',
                item: cloneDeep(newItems[j - 1])
            })
            j--
        }
    }
    while (i > 0) {
        result.unshift({
            type: 'deleted',
            item: cloneDeep(oldItems[i - 1])
        })
        i--
    }
    while (j > 0) {
        result.unshift({
            type: 'added',
            item: cloneDeep(newItems[j - 1])
        })
        j--
    }
    return result
}
